<?php

/**
* 	function get_caller_id ($AGI,$argv)
* 
*	Description 
*	function receives phone number passed by dialplan as argument and returns the name of the caller. 
*	on success return the status flag ${DP_STATUS} is set to 1 and caller name is set in ${DP_NAME}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_NAME} - Set to caller name retrieved 
*/

function get_caller_id ($AGI,$argv) {

        global $DP_username,$DP_password,$DP_url,$DEBUG;

	$phone = $argv[2];
	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=I&p1=".$phone;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index["STATUS"][0]]['value'] == "OK"){
		$name = $vals[$index["NAME"][0]]['value'];
		$AGI->set_variable("DP_NAME",$name);
		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","NameNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}
 
/**
* 	function get_carrier_info ($AGI,$argv)
* 
*	Description 
*	function receives phone number passed by dialplan as argument and return the carrier name, wireless
*	flag (y/n), sms gateway address and mms gateway address
*	on success return the status flag ${DP_STATUS} is set to 1 and carrier name, wireless flag, sms 
* 	gateway address and mms gateway address is set in ${DP_CARRIER_NAME},${DP_WIRELESS},${DP_SMS_ADDRESS}
*	, ${DP_MMS_ADDRESS}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_CARRIER_NAME} - Set to carrier name
*	${DP_WIRELESS} - Set to wireless flag (y/n)
*	${DP_SMS_ADDRESS} - Set to sms gateway address
*	${DP_MMS_ADDRESS} - Set to mms gateway address
*/

function get_carrier_info ($AGI,$argv) {

	global $DP_username,$DP_password,$DP_url;
	$phone = $argv[2];
	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=T&p1=".$phone;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index['STATUS'][0]]['value'] == "OK"){
		$wless = $vals[$index['WLESS'][0]]['value'];
		$carrier_name = $vals[$index['CARRIER_NAME'][0]]['value'];
		$sms_address = $vals[$index['SMS_ADDRESS'][0]]['value'];
		$mms_address = $vals[$index['MMS_ADDRESS'][0]]['value'];
		
		$AGI->set_variable("DP_WIRELESS",$wless);
		$AGI->set_variable("DP_CARRIER_NAME",$carrier_name);
		$AGI->set_variable("DP_SMS_ADDRESS",$sms_address);
		$AGI->set_variable("DP_MMS_ADDRESS",$mms_address);

		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","InformationNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}



/**
* 	function get_phone_append ($AGI,$argv)
* 
*	Description 
*	function accepts firstname,lastname,address,city,state,zip passed by dialplan as argument and returns
*	the associated phone number
*	on success return the status flag ${DP_STATUS} is set to 1 and phone number set to ${DP_PHONE}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_PHONE} - Set to phone number
*/

function get_phone_append ($AGI,$argv) {

	global $DP_username,$DP_password,$DP_url;
	$fname = $argv[2];
	$lname = $argv[3];
	$address = $argv[4];
	$city = $argv[5];
	$state = $argv[6];
	$zip = $argv[7];

	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=XP&fname=".urlencode($fname)."&lname=".urlencode($lname)."&address=".urlencode($address)."&city=".urlencode($city)."&state=".urlencode($state)."&zip=".$zip;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index['STATUS'][0]]['value'] == "OK"){
		$phone = $vals[$index['PHONE'][0]]['value'];
		
		$AGI->set_variable("DP_PHONE",$phone);
		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","InformationNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}

/**
* 	function get_email_append ($AGI,$argv)
* 
*	Description 
*	function accepts firstname,lastname,address,city,state,zip passed by dialplan as argument and returns
*	the associated email address
*	on success return the status flag ${DP_STATUS} is set to 1 and phone number set to ${DP_EMAIL}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_EMAIL} - Set to email id
*/


function get_email_append ($AGI,$argv) {

	global $DP_username,$DP_password,$DP_url;	
	$fname = $argv[2];
	$lname = $argv[3];
	$address = $argv[4];
	$city = $argv[5];
	$state = $argv[6];
	$zip = $argv[7];

	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=XE&firstname=".urlencode($fname)."&lastname=".urlencode($lname)."&address=".urlencode($address)."&city=".urlencode($city)."&state=".urlencode($state)."&zip=".$zip;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index['STATUS'][0]]['value'] == "OK"){
		$email = $vals[$index['EMAIL'][0]]['value'];
		
		$AGI->set_variable("DP_EMAIL",$email);
		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","InformationNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}


/**
* 	function get_reverse_phone ($AGI,$argv)
* 
*	Description 
*	function accepts a phone number passed by dialplan as argument and returns
*	the associated name and address
*	on success return the status flag ${DP_STATUS} is set to 1 and name is set to ${DP_NAME} and 
*	address is set to ${DP_ADDRESS}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_NAME} - Set to NAME
*	${DP_ADDRESS} - Set to address. This is a comma seprated value for address,city,state,zip
*/


function get_reverse_phone($AGI,$argv) {

	global $DP_username,$DP_password,$DP_url;	
	$phone = $argv[2];

	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=A&p1=".$phone;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index['STATUS'][0]]['value'] == "OK"){
		$fname = $vals[$index['FIRSTNAME'][0]]['value'];
		$lname = $vals[$index['LASTNAME'][0]]['value'];
		$m_initial = $vals[$index['M_INITIAL'][0]]['value'];
		$address = $vals[$index['ADDRESS'][0]]['value'];
		$city = $vals[$index['CITY'][0]]['value'];
		$state = $vals[$index['STATE'][0]]['value'];
		$zip = $vals[$index['ZIP'][0]]['value'];
		if($m_initial != "") {
			$fname = $fname." ".$m_initial; 
		}
		$name = $fname." ".$lname;

		$complete_addr = $address.",".$city.",".$state.",".$zip;

		$AGI->set_variable("DP_NAME",$name);
		$AGI->set_variable("DP_ADDRESS",$complete_addr);
		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","InformationNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}


/**
* 	function get_ip_geolocation ($AGI,$argv)
* 
*	Description 
*	function accepts a phone number passed by dialplan as argument and returns
*	the associated location and latitude longitude
*	on success return the status flag ${DP_STATUS} is set to 1 and location is set to ${DP_LOCATION} and 
*	latitude longitude is set to ${DP_LATITUDE_LONGITUDE}
* 	on failure return the status flag ${DP_STATUS} is set to 0 and failure reason is set in $DP_FAILURE 
*	
*	@params
* 	$AGI - AGI handler
* 	$argv - array contains list of arguments passed by dialplan
*	@return
*	${DP_STATUS} - set to 1 for success and 0 for failure
* 	${DP_FAILURE} - set to failure reason if $DP_STATUS is 0
*	${DP_LOCATION} - Set to location retrieved. This is a comma seprated value for country,state,city,$zip,timezone,dst
*	${DP_LATITUDE_LONGITUDE} - Set to latitude longitude. This is a comma seprated value for latitude,longitude
*/

function get_ip_geolocation($AGI,$argv) {
	global $DP_username,$DP_password,$DP_url;	
 	$ip = $argv[2];

	$url = $DP_url."?user=".$DP_username."&pass=".$DP_password."&api=P&p1=".$ip;
	list ($status,$response) = send_request($AGI,$url);
	if (!$status) {
		$AGI->exec("NoOp","ServerNotConnecting");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","ServerNotConnecting");
		return ($status);
	}
	$mainstring = parse_xml_response($response);
	$vals = $mainstring[1];
	$index = $mainstring[2];
	
	if (!$status) {
		$AGI->exec("NoOP","NoValidXML");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","NoValidXML");
		return ($status);
	}
	
	if ($vals[$index['STATUS'][0]]['value'] == "OK") {
		$country = $vals[$index['COUNTRY'][0]]['value'];
		$state = $vals[$index['STATE'][0]]['value'];
		$city = $vals[$index['CITY'][0]]['value'];
		$zip = $vals[$index['ZIPCODE'][0]]['value'];
		$latitude = $vals[$index['LATITUDE'][0]]['value'];
		$longitude = $vals[$index['LONGITUDE'][0]]['value'];
		$timezone = $vals[$index['TIMEZONE'][0]]['value'];
		$dst = $vals[$index['DST'][0]]['value'];

		$location_info = $country.",".$state.",".$city.",".$zip.",".$timezone.",".$dst;
		$latitude_longitude = $latitude.",".$longitude;
	
		$AGI->set_variable("DP_LOCATION",$location_info);
		$AGI->set_variable("DP_LATITUDE_LONGITUDE",$latitude_longitude);
		$AGI->set_variable("DP_STATUS","1");
		$AGI->set_variable("DP_FAILURE","");
		
	} else {
		$AGI->exec("NoOp","InformationNotFound");
		$AGI->set_variable("DP_STATUS","0");
		$AGI->set_variable("DP_FAILURE","InformationNotFound");
	}	
		
	return 0;
}

function send_request($AGI,$path){

	global $DEBUG;

	if($DEBUG) { 
	
		$msg = "\nURL PATH: $path\r\n";
		logToFile($msg,$AGI);
	}	

	$status = 1;
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL,$path);
	curl_setopt($ch, CURLOPT_FAILONERROR,1);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION,1);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	curl_setopt($ch, CURLOPT_TIMEOUT, 15);
	$retValue = curl_exec($ch);			 
	if(empty($retValue)) {
		$status = 0;
	}

	if($DEBUG) { 
	
		$msg = "\nRESPONSE : $retValue\n";
		logToFile($msg,$AGI);
	}	
	curl_close($ch);
	return array($status,$retValue);
}


function parse_xml_response ($xml) {

	$status = 1;
	$p = xml_parser_create();
	xml_parse_into_struct($p, $xml, $vals, $index);
	xml_parser_free($p);
	return array($status,$vals,$index);
}

function logToFile($msg,$AGI) {
	$date = date('Y_m_d');
	$filename = "/var/lib/asterisk/agi-bin/Data24-7/logs/log_$date.php";	
	$flag = 'w';
	if(file_exists($filename))  {
		$flag = 'a';
	}

	// open file
	$fd = fopen($filename, $flag);
	// append date/time to message
	$str = "[" . date("Y/m/d h:i:s", time()) . "] " . $msg; 
	// write string
	fwrite($fd, $str . "\n");
	// close file
	fclose($fd);

}

?>
